shithub: puzzles

Download patch

ref: a605a17d05acf4d981219c5e8db3def0b67a5c4a
parent: 6bce285027a9eacd50b92b5a26e1cd3c5c69a4ca
author: Simon Tatham <anakin@pobox.com>
date: Fri Jul 22 07:06:57 EDT 2005

James H profiled the new Same Game grid generator and discovered it
was spending 60% of its time in shuffle(). The purpose of the
shuffle() call was to go through a largish array in random order
until we found an element that worked, so there's no actual need to
shuffle the whole array every time and I only did it out of
laziness. So I now pick a random element each time I go round the
loop, meaning I save a lot of shuffling effort whenever the loop
terminates early (which is often). I get about a factor of two speed
improvement from this small change.

[originally from svn r6125]

--- a/samegame.c
+++ b/samegame.c
@@ -398,11 +398,6 @@
             if (n == 0)
                 break;		       /* we're done */
 
-            /*
-             * Shuffle the list.
-             */
-            shuffle(list, n, sizeof(*list), rs);
-
 #ifdef GENERATION_DIAGNOSTICS
             printf("initial grid:\n");
             {
@@ -420,13 +415,17 @@
 #endif
 
             /*
              * Now go through the list one element at a time and
              * actually attempt to insert something there.
              */
             while (n-- > 0) {
                 int dirs[4], ndirs, dir;
 
-                pos = list[n];
+                i = random_upto(rs, n+1);
+                pos = list[i];
+                list[i] = list[n];
+
                 x = pos % w;
                 y = pos / w;
@@ -420,13 +415,17 @@
 
             /*
-             * Now go through the list one element at a time and
-             * actually attempt to insert something there.
+             * Now go through the list one element at a time in
+             * random order, and actually attempt to insert
+             * something there.
              */
             while (n-- > 0) {